Package: clootl
Title: Fetch and Explore the Cornell Lab of Ornithology Open Tree of
        Life Avian Phylogeny
Version: 0.1.4
Authors@R: c(
  person("Eliot", "Miller", role=c("aut", "cre"), email = "clootlmaintainers@gmail.com"),
  person("Emily Jane", "McTavish", role=c("aut")),
  person("Luna L.", "Sanchez Reyes", role=c("ctb", "aut")))
Maintainer: Eliot Miller <clootlmaintainers@gmail.com>
URL: https://github.com/eliotmiller/clootl
BugReports: https://github.com/eliotmiller/clootl/issues
Depends: R (>= 4.3.0), ape
Imports: dplyr, RCurl, jsonlite
LazyData: true
LazyDataCompression: xz
Description: Fetches the Cornell Lab of Ornithology Open Tree of Life (clootl) tree in a specified taxonomy. Optionally prune it to a given set of study taxa. Provide a recommended citation list for the studies that informed the extracted tree. Tree generated as described in McTavish et al. (2024) <doi:10.1101/2024.05.20.595017>.
License: GPL-3
Encoding: UTF-8
RoxygenNote: 7.3.3
Suggests: rmarkdown, testthat (>= 3.0.0)
Config/testthat/edition: 3
NeedsCompilation: no
Packaged: 2026-04-23 17:13:30 UTC; luna
Author: Eliot Miller [aut, cre],
  Emily Jane McTavish [aut],
  Luna L. Sanchez Reyes [ctb, aut]
Repository: CRAN
Date/Publication: 2026-04-23 17:30:02 UTC
Built: R 4.6.0; ; 2026-04-26 12:12:46 UTC; unix

# Generated by roxygen2: do not edit by hand

export(extractTree)
export(getCitations)
export(get_avesdata_repo)
export(get_avesdata_repo_path)
export(sampleTrees)
export(set_avesdata_repo_path)
export(taxonomyGet)
import(ape)
importFrom(RCurl,postForm)
importFrom(dplyr,group_by)
importFrom(dplyr,summarize)
importFrom(jsonlite,fromJSON)
importFrom(jsonlite,toJSON)

# clootl

<!-- badges: start -->

<!-- usethis::use_cran_badge() -->

[![CRAN status](https://www.r-pkg.org/badges/version/clootl)](https://CRAN.R-project.org/package=clootl) [![CRAN RStudio mirror downloads](https://cranlogs.r-pkg.org/badges/clootl)](https://www.r-pkg.org/pkg/clootl)

<!-- badges: end -->

## An R package for interfacing with the clootl tree

Welcome to the clootl project! If you need a bird phylogeny, you've come to the right place. This is the place to be for downloading and learning how to use clootl.

#### What does clootl do?

Downloads the clootl phylogeny to your local machine and provides a set of functions for extracting and interacting with it. The most significant of these are `extractTree`, used for extracting a phylogeny pruned to a set of user-defined species, and `getCitations`, used for listing and quantifying the contribution of constituent phylogenies to these extracted trees.

#### What does clootl come with?

The package here, which you can also find over on [CRAN](https://CRAN.R-project.org/package=clootl), comes with a series of maximum clade credibility (MCC) trees. We anticipate that most users are going to want the current phylogeny in the current Clements taxonomy, and that is part of the "base" clootl download. This download also includes MCC trees for a number of older Clements taxonomies and tree topologies. We've been working on this project since 2020, maintaining these older taxonomies and topologies, but 2025 marks an inflection point in our pace of development. It will be challenging to produce the current topology in older taxonomies. We're not saying we won't do it, but we'd want to hear a clamoring of need for it before we invest the time. After this year we will retire pre-2025 taxonomies and pre-v1.6 tree topologies, and going forward intend to maintain 1-2 years' worth of previous taxonomy/phylogeny combinations at any given time.

While our approach in building the clootl trees glosses over the uncertainty associated with topology in the input trees, there is still an element of stochasticity in the resolution of polytomies, the placement of the missing taxa, and the time-scaling steps. We provide a "cloud" of complete, time-scaled phylogenies in a [separate download](https://github.com/eliotmiller/clootl/blob/master/examples/dataDownload.md), and clootl comes with functionality for integrating this download with existing tree manipulation functions.

#### How do I download clootl?

Easy, recommended:

``` r
library(devtools)
install_github("eliotmiller/clootl")
library(clootl)
```

Not recommended. The version posted to CRAN can lag behind what you'll find here, but if you insist, you can download clootl from CRAN:

``` r
install.packages("clootl")
library(clootl)
```

### Tutorials

-   [Getting Started](https://github.com/eliotmiller/clootl/blob/master/examples/intro.md)
-   [Downloading the Data Repository](https://github.com/eliotmiller/clootl/blob/master/examples/dataDownload.md)

.onAttach<-function(libname, pkgname){
    requireNamespace("utils")
    cit<-utils::citation(pkgname)
    txt<-paste(c(format(cit,"citation")),collapse="\n\n")
    txt<-paste(c(txt, "The current version of the Aves tree is v1.6.2025.
        Please specify the tree and taxonomy version used when citing this R package.
        When possible, cite all the original studies supporting your tree:
        These citations are acessible using getCitations(your_tree)"),collapse="\n\n")
    packageStartupMessage(txt)
}

#' Download the AvesData full repository
#' @description Pull down full AvesData repository to a working directory
#' @param path Path to download data zipfile to, and where it will be unpacked.  To download into your working directory, use "."
#' @param overwrite Default to `FALSE`. If path exists, will not re-download the data unless overwrite=`TRUE`.
#' @return No return value. This function is used to download the Aves Data repository.
#' @details Will download full data repo from https://github.com/McTavishLab/AvesData. 
#' This data is required to use [sampleTrees()] to sample from the distribution of dated trees,
#' or to access earlier versions of the complete tree. 
#' This function will download the data and set an environmental variable `AVESDATA_PATH` to the location of the data download.
#' When `AVESDATA_PATH` is set, the data_path in any clootl functions with a `data_path` argument will default to this value.
#' To manually set `AVESDATA_PATH` to the location of your downloaded AvesData repo use [set_avesdata_repo_path()]
#' @export
get_avesdata_repo <- function(path,
                              overwrite=FALSE){
  path <- normalizePath(path, winslash = "/")
  if (Sys.getenv("AVESDATA_PATH") != "" & (overwrite == FALSE)){
    message(paste("AVESDATA_PATH already set to:",
                 Sys.getenv("AVESDATA_PATH"),
                 "use overwrite = TRUE to download and overwite existing path."))
    return(invisible(1))
  }

  message("Downloading AvesDataLite repo from github (holds key files from large McTavishLab/AvesData repo). This may take several minutes depending on your connection.")
  url = "https://github.com/McTavishLab/AvesDataLite/archive/refs/heads/main.zip?raw=TRUE"
  old <- options() # save current options
  on.exit(options(old)) #Revert to original options on exit
  options(timeout=1000) # This file is big and can take a little while to download
  if (!file.exists(path)){
      stop("Directory to save AvesData not found:", path)
    }
  zipfilepath = file.path(path, 'AvesDataLite.zip')
  if (file.exists(zipfilepath) & (overwrite == FALSE)){
    message("File AvesDataLite.zip already exists. Use overwite = TRUE to download a new version")
  } else {
    utils::download.file(url, destfile = zipfilepath, mode="wb")
    stopifnot(file.exists(zipfilepath)) 
    stopifnot(file.exists(path)) 
    utils::unzip(zipfile = zipfilepath, exdir = path, overwrite=TRUE)
  }
  #  avesdata_path = paste(path, "/", "AvesDataLite-main", sep="")
  avesdata_path = file.path(path, 'AvesDataLite-main')
  avesdata_path <- normalizePath(avesdata_path, winslash = "/")
  # stopifnot(file.exists(avesdata_path)) 
  avesdata_path = path.expand(avesdata_path)
  # stopifnot(file.exists(exp_avesdata_path)) 
  set_avesdata_repo_path(avesdata_path, overwrite=overwrite)
  message("AvesDataLite repo downloaded and upzipped to:", avesdata_path)
  invisible(avesdata_path)
}

#' 
#' Get path to Aves Data folder
#' @description Get path to Aves Data folder, if set.
#' @return String - path to Aves Data folder, if set. Returns "" if not set.
#' @details  Based on https://github.com/CornellLabofOrnithology/auk/blob/main/R/auk-set-ebd-path.r
#' Use this function to check stored path to downloaded AvesData folder from https://github.com/McTavishLab/AvesData.
#' When `AVESDATA_PATH` is set, the data_path in any clootl functions with a `data_path` argument will default to this value.
#' @export
#' @examples
#' \dontrun{
#' get_avesdata_repo_path()
#' }
#' @export
get_avesdata_repo_path <- function(){
  path = Sys.getenv("AVESDATA_PATH")
  message(paste("AVESDATA_PATH set to:",
                 path,
                 "use set_avesdata_repo_path to change path."))
  return(path)
}


#' 
#' Set path to Aves Data folder
#' @description Set path to Aves Data folder already somewhere on your computer
#' @param path A character vector with the path to the Aves Data folder
#' @param overwrite Boolean, default to `FALSE`, does not overwrite an existing Aves Data folder. Set to `TRUE` to overwrite.
#' @param warn Boolean, default to `TRUE`, Warns if path does not exist. 
#' Set to FALSE and path="" to unset path.
#' @return No return value, called to set the path to the Aves Data folder.
#' @details  Based on https://github.com/CornellLabofOrnithology/auk/blob/main/R/auk-set-ebd-path.r
#' Use this function to manually set or update location of a downloaded AvesData folder from https://github.com/McTavishLab/AvesData.
#' When `AVESDATA_PATH` is set, the data_path in any clootl functions with a `data_path` argument will default to this value.
#' @export
#' @examples
#' \dontrun{
#' set_avesdata_repo_path("/home/ejmctavish/AvesData")
#' }
set_avesdata_repo_path <- function(path, overwrite = FALSE, warn = TRUE){
  if (!file.exists(path) & (warn == TRUE)){
      stop("Path not found: ", path)
    }
  if (path != ""){
    path <- normalizePath(path, winslash = "/", mustWork = warn)
  }
    if (overwrite) {
      # drop existing
      Sys.setenv(AVESDATA_PATH = path)
    } else {
      message(paste("AVESDATA_PATH already set to:",
                 Sys.getenv("AVESDATA_PATH"),
                 "use overwrite = TRUE to overwite existing path."))
  }
    
  
    message(paste("AVESDATA_PATH set to", path))
    # set AVESDATA_PATH for this session, so user doesn't have to reload
    Sys.setenv(AVESDATA_PATH = path)
    invisible(path)

 }

library(clootl)

test_that(desc = "extractTree works with scientific names", code = {
## Test sci name extract
    fulltree<-extractTree()
    expect_type( object = fulltree, type = "list")

    sci_name_spp <- c("Turdus migratorius",
                      "Setophaga dominica",
                      "Setophaga ruticilla",
                      "Sitta canadensis")
    expect_contains(fulltree[['tip.label']], sci_name_spp)



    sci_name_tree <- extractTree(species=sci_name_spp)
        expect_equal( length(sci_name_tree), 6 )
        expect_type( object = sci_name_tree, type = "list")
        expect_equal( sci_name_tree$Nnode, 3 )
        expect_contains(sci_name_tree[['tip.label']], sci_name_spp)

#    sci_name_tree14_2021 <- extractTree(species=sci_name_spp, taxonomy_year=2021)
#    sci_name_tree14_2022 <- extractTree(species=sci_name_spp, taxonomy_year=2022)
    #sci_name_tree14_2023 <- extractTree(species=sci_name_spp, version=1.4, taxonomy_year=2023)
    sci_name_tree16_2025 <- extractTree(species=sci_name_spp, version=1.6, taxonomy_year=2025)

        # expect_identical(sci_name_tree, sci_name_tree14_2023)
#        expect_equal( length(sci_name_tree14_2021), 6 )
        expect_contains(sci_name_tree16_2025[['tip.label']], sci_name_spp)
        expect_equal( length(sci_name_tree16_2025), 6 )
        expect_contains(sci_name_tree16_2025[['tip.label']], sci_name_spp)

    cites <- getCitations(sci_name_tree)

        expect_equal( length(cites), 4 )
        # expect_equal( length(cites[['study']]), 13 )

#    cites2021 <- getCitations(sci_name_tree14_2021)
#    cites2022 <- getCitations(sci_name_tree14_2022)
    # cites2023 <- getCitations(sci_name_tree14_2023)
#    expect_identical(cites, cites2021)
#    expect_identical(cites, cites2022)
    # expect_identical(cites, cites2023)

})



test_that(desc = "extractTree works with code names", code = {
## Test code extract

  fulltree_code<-extractTree(label_type="code")
  expect_type( object = fulltree_code, type = "list")


  code_spp <- c("amerob", "canwar", "reevir1", "yerwar", "gockin")
  expect_contains(fulltree_code[['tip.label']], code_spp)

  code_tree <- extractTree(species=code_spp,
                         label_type="code")
  expect_identical(sort(code_tree[['tip.label']]), sort(code_spp))
  bad_code_spp <- c("amerob", "canwar", "reevir1", "yerwar", "error")
  expect_error(extractTree(species=bad_code_spp,
                         label_type="code"))

  ## Recover using FORCE
  pruned <- extractTree(species=bad_code_spp,
                        label_type="code",
                        force=TRUE)
  expect_equal(length(pruned$tip.label), 4)

})


test_that(des = "sampleTrees errors works", code = {
  expect_error(sampleTrees(species=c("Turdus migratorius",
                        "Setophaga dominica",
                        "Setophaga ruticilla",
                        "Sitta canadensis"),
              label_type="code",
              taxonomy_year="1999",
              version="1.6",
              count=100,
              data_path=FALSE))
  expect_error(sampleTrees(species=c("Turdus migratorius",
                        "Setophaga dominica",
                        "Setophaga ruticilla",
                        "Sitta canadensis"),
              label_type="code",
              taxonomy_year="1999",
              version="11",
              count=100,
              data_path=FALSE))
  # expect_true(length(xx)==100)
  # expect_s3_class(xx, "multiPhylo")
})

test_that(des = "sampleTrees fails with no data download", code = {
  expect_error(sampleTrees(species=c("Turdus migratorius",
                        "Setophaga dominica",
                        "Setophaga ruticilla",
                        "Sitta canadensis"),
              label_type="code",
              taxonomy_year="2025",
              version="1.6",
              count=100,
              data_path=FALSE))
  # expect_true(length(xx)==100)
  # expect_s3_class(xx, "multiPhylo")
})

test_that(desc = "errors work on extractTree", code = {
  bad_sci_name_spp <- c("Turdus migratorius",
                        "Setophaga dominica",
                        "Setophaga ruticilla",
                        "Error badspecies")

  expect_error(extractTree(species="all_species",
                           label_type="scientific",
                           taxonomy_year=1999))

  expect_error(extractTree(species="all_species",
                           label_type="scientific",
                           version="0.5"))

  expect_error(extractTree(species="all_species",
                           label_type="scientific",
                           taxonomy_year=2025,
                           version="1.2"))

  expect_error(extractTree(species="all_species",
                           label_type="unknown"))

  expect_error(extractTree(species=bad_sci_name_spp,
                           label_type="scientific"))

  pruned <- extractTree(species=bad_sci_name_spp,
                        label_type="scientific",
                        force=TRUE)
  expect_equal(length(pruned$tip.label), 3)
})

test_that(desc = "extractTree gets the full tree", code = {
  xx <- extractTree()
  expect_s3_class(xx, "phylo")
  xx <- extractTree(label_type = "code")
  expect_s3_class(xx, "phylo")
})

test_that(desc = "errors work on taxonomyGet", code = {
  expect_error(taxonomyGet(taxonomy_year=1999))
})

## Tests below here rely on local data download and should be skipped on cran
test_that(desc = "several data download requiring tests (skip on cran)", code = {
#  skip_on_cran()
#  skip_if_offline()
  old_data_path <- get_avesdata_repo_path()
  tmpdir_path = tempdir()
  # download folder and set path
  get_avesdata_repo(tmpdir_path, overwrite=TRUE) 
  path = get_avesdata_repo_path()

  test_spp = c("Turdus migratorius",
               "Setophaga dominica",
               "Setophaga ruticilla",
               "Sitta canadensis")


  ## Do not pass in data path 
  tree1_5_2022_env <- extractTree(species=test_spp,
                                  version = 1.5,
                                  taxonomy_year=2022)
  expect_equal(length(tree1_5_2022_env$tip.label), 4)

  cites <- getCitations(tree1_5_2022_env, version=1.5)

  expect_equal( length(cites), 4 )

  ## Pass in data path 
  tree1_5_2022_no_env <- extractTree(species=test_spp,
                            version = 1.5,
                            taxonomy_year=2022,
                            data_path=path)
  expect_equal(length(tree1_5_2022_no_env$tip.label), 4)
  expect_identical(tree1_5_2022_no_env, tree1_5_2022_env)


  ## Tree Get from file
  fulltree_1_4_2022 <- clootl:::treeGet(version = 1.4,
                                        taxonomy_year=2022,
                                        data_path=path)
  expect_equal(length(fulltree_1_4_2022$tip.label), 10906)

  ## Taxonomy Get from file
  taxonomy_2021 <- clootl:::taxonomyGet(taxonomy_year=2021,
                                        data_path=path,
                                        from_file=TRUE)
  expect_equal(dim(taxonomy_2021), c(10824, 17))


  xx <- sampleTrees(species=test_spp,
                    label_type="scientific",
                    taxonomy_year="2025",
                    version="1.6",
                    count=100,
                    data_path=path)
  expect_true(length(xx)==100)
  expect_s3_class(xx, "multiPhylo")

  set_avesdata_repo_path(old_data_path, overwrite=TRUE, warn=FALSE)
  })

#' A complex data store used in the package.
#'
#' @description A dataset containing taxonomy files, summary phylogenies, constituent study
#' information, and other data needed for the package to function properly.
#' @format List of csv files, phylogenies, and other data components.
#' @source \url{https://github.com/eliotmiller/clootl}
#' @details
#' The data object, \code{clootl_data}, stores the most up-to-date stable version of the
#' tree mapped to each of the different taxonomy years, the annotations of how each
#' study contributed to the tree, the citation information for each study that
#' contributed to the tree, the taxonomy crosswalks for different years, and
#' some other variables.
#'
#' The structure of the data store (a list) is as follows:
#'
#' \describe{
#'   \item{\code{clootl_data$taxonomies}}{
#'     A list of data frames. Each element corresponds to a taxonomy year:
#'     \itemize{
#'       \item \code{year2025}
#'       \item \code{year2024}
#'       \item \code{year2023}
#'       \item \code{year2022}
#'       \item \code{year2021}
#'     }
#'     These originate as CSV files linking the Clements taxonomy
#'     for each of these years to OTT ids, Avibase ids, and other bird taxonomies
#'     (see README of \url{https://github.com/McTavishLab/AvesData}).
#'   }
#'
#'   \item{\code{clootl_data$trees}}{
#'     \describe{
#'       \item{\code{summary.trees}}{
#'         Phylo objects of complete dated trees mapped to the Clements taxonomy year:
#'         \itemize{
#'           \item \code{year2025}
#'         }
#'         These are generated from \code{summary_dated_clements.nex}
#'         (see \url{https://github.com/McTavishLab/AvesData} README).
#'       }
#'       \item{\code{annotations}}{
#'         Complete annotations of the OpenTree synthetic tree for this version,
#'         used to determine appropriate subtree citations.
#'       }
#'     }
#'   }
#'
#'   \item{\code{clootl_data$study_info}}{
#'     A mapping of OpenTree study ids to full citations. Used with annotations to
#'     generate appropriate citations for trees and subtrees.
#'   }
#'
#'   \item{\code{clootl_data$versions}}{
#'     A character vector of all possible tree versions. To access older versions,
#'     download the data repository using \code{get_avesdata_repo()}.
#'   }
#'
#'   \item{\code{clootl_data$tax_years}}{
#'     A character vector of all available taxonomies. The current tree version is
#'     mapped to each of these taxonomies, along with crosswalks linking the Clements
#'     taxonomy for each year to other identifiers.
#'   }
#' }
#'
#' This data object is generated using the following code:
#'
#' \preformatted{
#' clootl_data = list()
#'
#' clootl_data$versions <- c("1.2","1.3","1.4","1.5","1.6")
#' clootl_data$tax_years <- c("2021","2022","2023","2024", "2025")
#'
#' 
#' clootl_data$combinations <- c(c(1.2, 2021),
#'                               c(1.2, 2022),
#'                               c(1.2, 2023),
#'                               c(1.3, 2021),
#'                               c(1.3, 2022),
#'                               c(1.3, 2023),
#'                               c(1.4, 2021),
#'                               c(1.4, 2022),
#'                               c(1.4, 2023),
#'                               c(1.5, 2021), <--- this will have every tip in AVONET
#'                               c(1.5, 2022),
#'                               c(1.5, 2023),
#'                               c(1.5, 2024),
#'                               c(1.6, 2025)) 
#'
#' fullTree2025 <- treeGet("1.6","2025", data_path="~/projects/otapi/AvesData")
#' clootl_data$trees$`Aves_1.6`$summary.trees$year2025 <- fullTree2025
#'
#' tax2021 <- taxonomyGet(2021, data_path="~/projects/otapi/AvesData")
#' tax2022 <- taxonomyGet(2022, data_path="~/projects/otapi/AvesData")
#' tax2023 <- taxonomyGet(2023, data_path="~/projects/otapi/AvesData")
#' tax2024 <- taxonomyGet(2024, data_path="~/projects/otapi/AvesData")
#' tax2025 <- taxonomyGet(2025, data_path="~/projects/otapi/AvesData")
#'
#' clootl_data$taxonomies$year2021 <- tax2021
#' clootl_data$taxonomies$year2022 <- tax2022
#' clootl_data$taxonomies$year2023 <- tax2023
#' clootl_data$taxonomies$year2024 <- tax2024
#' clootl_data$taxonomies$year2025 <- tax2025
#'
#'
#' annot_filename <- "~/projects/otapi/AvesData/Tree_versions/Aves_1.6/OpenTreeSynth/annotated_supertree/annotations.json"
#' all_nodes <- jsonlite::fromJSON(txt=annot_filename)
#' clootl_data$trees$Aves_1.6$annotations <- all_nodes
#'
#' studies <- c()
#' for (inputs in all_nodes$source_id_map) studies <- c(studies, inputs$study_id)
#' studies <- unique(studies)
#' study_info <- clootl:::api_studies_lookup(studies)
#'
#' clootl_data$study_info <- study_info
#' save(clootl_data, file="~/projects/otapi/clootl/data/clootl_data.rda", compress="xz")
#' }
"clootl_data"

utils::globalVariables(c("clootl_data"))


#' Identify contributing studies
#'
#' Quantify the contribution of studies informing an extracted tree,
#' and obtain DOI and citation information for those studies.
#'
#' @param tree A phylogeny obtained from extractTree (see details).
#' @param data_path Default to FALSE. If you are gathering citations for an
#' older version of the tree than the current one packed in the data object, you will have
#' already downloaded the data repo in order to generate that tree.
#' The data is available at https://github.com/McTavishLab/AvesData.
#' If you have manually downloaded the repo, use data_path= the path to the download location.
#' Alternately, you can download the full data repo using [get_avesdata_repo()]. 
#' This approach will download the data and set an environmental variable AVESDATA_PATH. 
#' When AVESDATA_PATH is set, the data_path argument will default to this value.
#' To manually set AVESDATA_PATH to the location of your downloaded AvesData repo 
#' use [set_avesdata_repo_path()]
#' @param version The version of the tree used in extract tree. Default to the most recent
#' version of the tree. and can be passed as a character string or as numeric. If an alternate 
#' version was used to create the tree this function may fail or give incomplete or incorrect 
#' citation information.
#'
#' @details The function will determine
#' what proportion of nodes in your phylogeny are supported by each study that goes into
#' creating the final clootl tree. We use 'supported by' in the sense described in
#' Redelings and Holder, PeerJ (2017) https://peerj.com/articles/3058/, and as shown in the
#' tree.opentreeoflife.org tree viewer. We normalize these values to
#' a percentage of internal nodes in the target tree supported by each study. 
#' In any resulting publication, please cite both the
#' synthetic tree (McTavish et al. 2025), clootl (Miller et al. 2025) and 
#' "all" the trees/DOIs that contributed to 
#' your phylogeny. That said, we are well aware of citation and word count 
#' limits that plague modern publishing, and for this reason we quantify the
#' contribution of each study; depending on your phylogeny,
#' it is very possible that one or two studies contributed the majority of information.
#' This function relies on the phylogenetic synthesis information directly, 
#' and is agnostic to taxonomy version.
#'
#' @return A dataframe of the percent of internal nodes supported by a given study, as well
#' as the DOI of that study. The proportion of taxa in the tree supported by taxonomic
#' addition only is also included in the dataframe.
#'
#' @author Eliot Miller, Emily Jane McTavish
#'
#' @export
#'
#' @import ape
#' @importFrom jsonlite toJSON fromJSON
#' @importFrom RCurl postForm
#' @importFrom dplyr group_by summarize
#'
#' @examples
#'\donttest{#pull the taxonomy file out
#' data(clootl_data)
#' tax <- clootl_data$taxonomies$year2025
#' ls(tax)
#' #simulate extracting a tree for a particular family
#' temp <- tax[tax$FAMILY=="Rhinocryptidae (Tapaculos)",]
#' spp <- temp$SCI_NAME
#'
#' #get your tree
#' prunedTree <- extractTree(species=spp, label_type="scientific",
#'    taxonomy_year=2025, version="1.6")
#'
#' #get your citation DF
#' yourCitations <- getCitations(tree=prunedTree)}
getCitations <- function(tree, version=1.6, data_path=FALSE) {##ToDO not hardcode?
  #pull the node labels out. count any (character instances of) NA, as this should
  #be the contribution of taxonomic additions and drop any NAs
  nodesToQuery <- tree$node.label
  taxonomyNodes <- sum(nodesToQuery == "NA")
  nodesToQuery <- nodesToQuery[nodesToQuery != "NA"]

  if (!is.element(version, clootl_data$versions)){
    stop("version not recognized: ", version)
  }

  if (data_path==FALSE){
          data_path = Sys.getenv('AVESDATA_PATH')
      }

  if (version == 1.6){ ##not hardcode
      utils::data("clootl_data")
      all_nodes <- clootl_data$trees$`Aves_1.6`$annotations ##ToDO not hardcode?
  } 
  else if (!file.exists(data_path)){##ToDO not hardcode?
    stop("GetCitations for anything other than the current tree requires an Aves Data download.
      Currently get citations needs you to run get_avesdata_repo() first
      or provide a path to the data repo using data_path=")
    }
  else{
      message("Using data download for annotations in")
      filename <- paste(data_path, '/Tree_versions/Aves_', version, '/OpenTreeSynth/annotated_supertree/annotations.json', sep='')
      message(filename)
      if (!file.exists(filename)){
          stop("annotations file not found at: ", filename)
          }
      all_nodes <- jsonlite::fromJSON(txt=filename)
  }

  trees <- c()
  studies<-c()
  ##
  for (node in nodesToQuery){
    node_trees <- names(all_nodes$nodes[[node]]$supported_by)
    trees <- c(node_trees, trees)
    node_studies <- (unique(sub("@.*", "", node_trees)))
    studies <-c(node_studies, studies)
  }
  if (length(studies) <= 1) {
      stop("Error: no studies found")
      }

  ## So this just lists all the trees, and all the studies...
  finalCounts <- as.data.frame(table(studies))
  colnames(finalCounts) <- c("study", "counts")

  study_info <- clootl_data$study_info

  dois <- c()
  refs <- c()
  for(i in 1:length(finalCounts$study))
  {
    study_id <- finalCounts$study[i]
    dois[i] <- study_info[study_info$study_id==study_id,'doi']
    refs[i]<- study_info[study_info$study_id==study_id,'reference']
      }
    
   #plug the dois in
  finalCounts$reference <- refs
  finalCounts$doi <- dois

  #add a row for contributions by taxonomy
  toBind <- data.frame(study="Taxonomic additions", counts=taxonomyNodes,
                       reference="Miller et al.",
                       doi="https://github.com/eliotmiller/addtaxa")
  finalCounts <- rbind(finalCounts, toBind)

  #order in decreasing order, strip row names, and return
  finalCounts <- finalCounts[order(finalCounts$counts, decreasing=TRUE),]
  row.names(finalCounts) <- NULL

  #finally divide through by the number of internal nodes
  finalCounts$contribution <- (finalCounts$counts/tree$Nnode)*100

  #drop the raw counts and return
  finalCounts$counts <- NULL
  finalCounts
}


## This is for internal use to set up the study_info data object
api_studies_lookup <- function(studies){
  study_info <- data.frame(matrix(vector(), 0, 3,
                          dimnames=list(c(), c("study_id", "reference", "doi"))),
                          stringsAsFactors=F)

  studies <- as.vector(studies)
  for(i in 1:length(studies))
  {
    #again, define some convenience variables
    url <- "https://api.opentreeoflife.org/v3/studies/find_studies"
    headers <- c('Content-Type' = 'application/json')
    body <- jsonlite::toJSON(list("property"="ot:studyId",
                                  "value"=studies[i],
                                  "verbose"="true"),
                                   auto_unbox=TRUE)

    response <- RCurl::postForm(uri = url,
                         .opts = list(
                           postfields = body,
                           httpheader = headers
                         ))

    #having trouble with weird end of line symbols
    response <- gsub("\n", "", response)

    # Parse the response 
    parsedJSON <- jsonlite::fromJSON(response)
    doi <- parsedJSON$matched_studies$`ot:studyPublication`
    if(is.null(doi))
    {
        doi <- NA
      }
    reference <- parsedJSON$matched_studies$`ot:studyPublicationReference`
      if(is.null(reference))
      {
        reference <- NA
      }

    study_info <- rbind(study_info, data.frame(study_id =studies[i],
                                              reference = reference, 
                                              doi = doi))
    }
    return(study_info)
  }

utils::globalVariables(c("clootl_data"))

#' Extract a cloud of trees from the complete Avian Phylogeny for a set of species
#'
#' @param species A character vector either of scientific names (directly as they come out of the
#' eBird taxonomy, i.e. without underscores) or of six-letter eBird species codes. Any elements of
#' the species vector that do not match a species-level taxon in the specified eBird taxonomy
#' will result in an error. Default is set to "all_species".
#' @param label_type Either "scientific" or "code". Default is set to "scientific".
#' @param taxonomy_year The eBird taxonomy year the tree should be output in. Current options
#' include 2021, 2022, and 2023. Both numeric and character inputs are acceptable here. Any value
#' aside from these years will result in an error. Default is set 2023.
#' @param data_path Default to FALSE. Data_path is not necessary if the path has been stored in the
#' environment, as occurs when the data was downloaded using get_avesdata_repo. 
#' Otherwise, pass in the path to the directory where the AvesData repo was downloaded. 
#' 
#' @param version The desired version of the tree. Default to the most recent
#' version of the tree. Other versions available are listed in data object as clootl_data$versions
#' Verson can be passed as a character string or as numeric.
#' @param count Work in progress, can only sample 100 for now. Eventually: The desired number of sampled trees.
#'
#' @details This function first ensures that the requested output species overlap with species-level
#' taxa in the requested eBird taxonomy. If they do not, the function will error out. The onus is
#' on the user to ensure the requested taxa are valid. This is critical to ensure no unexpected
#' analysis hiccups later--you don't want to find out many steps later that your dataset doesn't
#' match your phylogeny. The eBird database is currently (as of Mar 2025) in 2024 taxonomy.
#' Trees available in 2024 taxonomy will be available by June 2025. The 2025 taxonomy will be released
#' to the public in October or November 2025. The intention is to release a tree in 2025 taxonomy
#' concurrently with the publication of the taxonomy itself.
#'
#' @return A set of phylogenies determined in `count` of the specified taxa in the specified eBird taxonomy version and clootl
#' tree version.
#'
#' @author Eliot Miller, Luna Sanchez Reyes, Emily Jane McTavish
#'
#' @export
#'
#' @import ape
#'
#' @examples
#' if (Sys.getenv("AVESDATA_PATH") != "") {
#'   ex2 <- sampleTrees(species=c("Turdus migratorius",
#'                              "Setophaga dominica",
#'                              "Setophaga ruticilla",
#'                              "Sitta canadensis"))
#'  }
sampleTrees <- function(species="all_species",
                        label_type="scientific",
                        taxonomy_year="2025",
                        version="1.6",
                        count=100,
                        data_path=FALSE)
{
  label_type <- match.arg(label_type,c('code','scientific'))
  utils::data("clootl_data")

  version <- as.character(version)
  if (!is.element(version, clootl_data$versions)){
    stop("version not recognized: ", version) ## TODO print out actual list
  }

  taxonomy_year <- as.character(taxonomy_year)
  if (!is.element(taxonomy_year, clootl_data$tax_years)){
    stop("Requested year currently unavailable")
  }


  if((Sys.getenv('AVESDATA_PATH') == "") & (data_path==FALSE)){
      stop("AvesData repo not found.
        To sample across trees you need to download the Aves data repo.
      Either run get_avesdata_repo()
      or set path to Aves Data repo using set_avesdata_repo(path),
      or use the argument data_path = AvesData-path")
    }
    else {
        data_path = Sys.getenv('AVESDATA_PATH')
    }

  if(label_type=="code")
  {
   stop("sampling over trees currently only works with full names and label='scientific'")
  }
  if(count!=100)
  {
   stop("sampling over trees currently only returns all 100 trees.")
  }
  tax <- taxonomyGet(taxonomy_year)

  species <- as.list(species)
  #if species is set to all.species, redefine species as the full set of taxa

  if(species[1]=="all_species" & label_type=="scientific")
  {
    species <- tax$SCI_NAME
  }

  else
  {
    species <- species
  }


  if(label_type=="scientific")
  {
    #identify mismatches
    issues <- setdiff(species, tax$SCI_NAME)

    #if there are any, throw an error
    if(length(issues) > 0)
    {
      message("Some of your provided species codes do not match with codes 
        in the requested year's eBird taxonomy")
      stop(paste(issues, collapse = "\n"))
    }

    #else plug in underscores
    else
    {
      species <- sub(" ", "_", species)
    }
  }

  else
  {
    stop("Sample trees only works with label_type 'scientific'")
  }
    sample_tree_filename <- paste(data_path,
                               "/Tree_versions/",
                               "Aves_",
                               version,
                               "/Clements",
                               as.character(taxonomy_year),
                               "/dated_rand_sample_clements.tre",
                               sep='')
    message("Sampling trees from:")
    message(sample_tree_filename)
    if (!file.exists(sample_tree_filename)){
      stop("Tree set file:", sample_tree_filename,
        " is not found. This version may not be available for this taxonomy year or you may need to update your AvesData repo using get_avesdata_repo(overwrite=TRUE)")
    } else {
    fullTreeSet <- read.tree(sample_tree_filename,skip=1)
  }
  #now prune the tree and extract. if species is the full set, no pruning will occur
  pruned <- dropTipMultiPhylo(fullTreeSet, setdiff(fullTreeSet[[1]]$tip.label, species))
  pruned
}


# from http://blog.phytools.org/2020/06/pruning-tips-from-multiphylo-object.html
dropTipMultiPhylo<-function(phy, tip, ...){
    if(!inherits(phy,"multiPhylo"))
        stop("phy is not an object of class \"multiPhylo\".")
    else {
        trees<-lapply(phy,drop.tip,tip=tip,...)
        class(trees)<-"multiPhylo"
    }
    trees
}

utils::globalVariables(c("clootl_data"))


#' Extract a complete or pre-pruned phylogeny from the clootl datastore
#'
#' @description This function extracts one or more phylogenies in the desired taxonomy and tree version.
#' It defaults to the pre-packaged summary trees, but can also be used to extract sets of
#' phylogenies expressing uncertainty, once they have been downloaded from the online repository.
#' @param species A character vector either of scientific names (directly as they come out of the
#' eBird taxonomy, i.e. without underscores) or of six-letter eBird species codes. Any elements of
#' the species vector that do not match a species-level taxon in the specified eBird taxonomy
#' will result in an error. eBird taxonomy files can be accessed using [taxonomyGet()]. Default is set to "all_species".
#' @param label_type Either "scientific" or "code". Default is set to "scientific".
#' @param taxonomy_year The eBird taxonomy year the tree should be output in. Current options
#' are 2021-2024. Both numeric and character inputs are acceptable here. Any value
#' aside from these years will result in an error. Default is most recent year.
#' @param data_path Default to FALSE. If a summary, dated tree is desired, this is sufficient
#' and does not need to be modified. However, if a user wishes to extract a set of complete
#' dated trees, for example to iterate an analysis across a cloud of trees, or to use an
#' older version of the tree than the current one packed in the data object, this function
#' can also accept a path to the downloaded set of trees. If you have already downloaded the AvesData repo
#' available at https://github.com/McTavishLab/AvesData use data_path= the path to the download location.
#' Alternately, you can download the full data repo using [get_avesdata_repo()]. This approach will download the data and
#' set an environmental variable AVESDATA_PATH. When AVESDATA_PATH is set, the data_path will default to this value.
#' To manually set AVESDATA_PATH to the location of your downloaded AvesData repo use [set_avesdata_repo_path()]
#' @param version The desired version of the tree. Default to the most recent
#' version of the tree. Other versions available are listed in clootl_data$versions and can be
#' passed as a character string or as numeric.
#' @param force Default to FALSE.  If FALSE a tree will be returned only if there is an exact match in the tree
#' to all species requested. This function first ensures that the requested output species overlap with species-level
#' taxa in the requested eBird taxonomy. If they do not, the function will error out. The onus is
#' on the user to ensure the requested taxa are valid. This is critical to ensure no unexpected
#' analysis hiccups later--you don't want to find out many steps later that your dataset doesn't
#' match your phylogeny.
#' If force=TRUE even if there is is not a match to all taxa in the requested species list, a tree will be
#' returned for the species that do match.
#'
#'
#' @return One or more phylogenies of the specified taxa in the specified eBird taxonomy version and clootl
#' tree version.
#'
#' @author Eliot Miller, Luna Sanchez Reyes, Emily Jane McTavish
#'
#' @export
#'
#' @import ape
#'
#' @examples
#' ex1 <- extractTree(species=c("amerob", "canwar", "reevir1", "yerwar", "gockin"),
#'    label_type="code")
#' ex2 <- extractTree(species=c("Turdus migratorius",
#'                              "Setophaga dominica",
#'                              "Setophaga ruticilla",
#'                              "Sitta canadensis"),
#'    label_type="scientific",
#'    taxonomy_year="2025",
#'    version="1.6")
#'
extractTree <- function(species="all_species",
                        label_type="scientific",
                        taxonomy_year=2025,
                        version="1.6",
                        data_path=FALSE,
                        force=FALSE){
  label_type <- match.arg(label_type,c('code','scientific'))
  utils::data("clootl_data")


  version <- as.character(version)
  if (!is.element(version, clootl_data$versions)){
    stop("Tree version not recognized: ", version) ## TODO print out actual list from clootl_data$versions
  }

  taxonomy_year <- as.character(taxonomy_year)
  if (!is.element(taxonomy_year, clootl_data$tax_years)){
    stop("Requested taxonomy year unavailable: ", taxonomy_year)
  }

  combo = paste(version, "_tax", taxonomy_year, sep="")
  if (!is.element(combo, clootl_data$combinations)){
      stop(paste(combo, "This combination of tree version and taxonomy year is not available.
        Available combinations are:\n", paste(clootl_data$combinations, collapse=",\n")))
  }


  if (!is.element(label_type, c("code", "scientific")))
  {
    stop("label_type must be set to either 'code' or 'scientific'")
  }

  if((Sys.getenv('AVESDATA_PATH') == "") & (data_path==FALSE) & (version!='1.6')){
      stop("Only tree version 1.6 taxonomy year 2025 is currently packaged with clootl.
      To get alternate tree versions and taxonomy years, run get_avesdata_repo()
      or set path to Aves Data repo using set_avesdata_repo(path),
      or use the argument data_path = AvesData-path")
    }

  if((version=='1.6') & (taxonomy_year==2025)){
    utils::data("clootl_data")
    taxonomyYear <- paste("year", taxonomy_year, sep="")
    tax <- clootl_data$taxonomies[[`taxonomyYear`]]
    fullTree <- clootl_data$trees[[paste("Aves_", version, sep="")]]$summary.trees[[taxonomyYear]]
  }
  else{
      tax <- taxonomyGet(taxonomy_year, data_path)
      fullTree <- treeGet(version, taxonomy_year, data_path)
  }


  species <- as.list(species)
  if(species[1]=="all_species" & label_type=="scientific")
  {
    return(fullTree)
  }

  #if species is set to all.species, redefine species as the full set of taxa
  if(label_type=="scientific")
  {
    #identify mismatches

    issues <- setdiff(species, tax$SCI_NAME)
    species <- intersect(tax$SCI_NAME, species)
    #if there are any, throw an error
     if(length(issues) > 0 & force==FALSE)
    {
      message("Some of your provided species names do not match with names in the requested year's eBird taxonomy:")
      message(paste(length(issues),"names did not match.", length(species), "names did match", sep=" "))
      message("These names did not match:")
      message(paste(issues, collapse = "\n"))
      stop("Re-run with extractTree force=TRUE to generate a tree just for the names that do match.")
    }
      else if(length(issues) > 0 & force==TRUE)
    {
      message("Some of your provided species names do not match with names in the requested year's eBird taxonomy:")
      message(paste(length(issues),"names did not match.", length(species), "names did match", sep=" "))
      message("These names did not match:")
      message(paste(issues, collapse = "\n"))
      message("Argument force=TRUE, returning a tree for the species that match.")
    }

  }
  #check whether the input species are valid
  if(label_type=="code")
  {
     if(species[1]=="all_species")
     {
       species <- tax$SPECIES_CODE
      }

    #identify mismatches
    issues <- setdiff(species, tax$SPECIES_CODE)
    species <- intersect(tax$SPECIES_CODE, species)

    #if there are any, throw an error
    if(length(issues) > 0 & force==FALSE)
    {
      message("Some of your provided species codes do not match with codes in the requested year's eBird taxonomy:")
      message(paste(length(issues),"codes did not match.", length(species), "codes did match", sep=" "))
      message("These codes did not match:")
      message(paste(issues, collapse = "\n"))
      stop("Re-run with extractTree force=TRUE to generate a tree just for the names that do match.")
    }
    if(length(issues) > 0 & force==TRUE)
    {
      message("Some of your provided species names do not match with names in the requested year's eBird taxonomy:")
      message(paste(length(issues),"names did not match.", length(species), "names did match", sep=" "))
      message("These names did not match:")
      message(paste(issues, collapse = "\n"))
      message("Argument force=TRUE, returning a tree for the species that match.")
    }
      ## Prune to requested species
      #swap the scientific names for species codes
      newNames <- data.frame(order=1:length(fullTree$tip.label), orig=fullTree$tip.label)
      #merge and re-sort
      newNames <- merge(newNames, tax[,c("SCI_NAME","SPECIES_CODE")], by.x="orig", by.y="SCI_NAME")
      newNames <- newNames[order(newNames$order),]

      #swap names
      fullTree$tip.label <- newNames$SPECIES_CODE
    }


  #now prune the tree and extract. if species is the full set, no pruning will occur
  pruned <- drop.tip(fullTree, setdiff(fullTree$tip.label, species))

  message(paste("This extracted tree is from version ", version, " and taxonomy year ", taxonomy_year, ".
    Please cite the version, clootl, and the contributing studies using getCitations(tree, version).", sep=""))
  return(pruned)
}


#' Load a bird taxonomy into the R environment
#'
#' @description `taxonomyGet` either reads a taxonomy file and loads it
#' as a `data frame`, or loads the default taxonomy data object.
#'
#' @inheritParams extractTree
#' @param data_path Default to FALSE. If a summary, dated tree is desired, this is sufficient
#' and does not need to be modified. However, if a user wishes to extract a set of complete
#' dated trees, for example to iterate an analysis across a cloud of trees, or to use an
#' older version of the tree than the current one packed in the data object, this function
#' can also accept a path to the downloaded set of trees. If you have already downloaded the AvesData repo
#' available at https://github.com/McTavishLab/AvesData use data_path= the path to the download location.
#' Alternately, you can download the full data repo using [get_avesdata_repo()]. This approach will download the data and
#' set an environmental variable AVESDATA_PATH. When AVESDATA_PATH is set, the data_path will default to this value.
#' To manually set AVESDATA_PATH to the location of your downloaded AvesData repo use [set_avesdata_repo_path()]
#' @param from_file Default to FALSE. If TRUE forces taxonomyGet to use a local copy of the the taxonomy.
#' This is useful for testing changes and/or updating the clootl_data object.
#' @details This will return a data object that has the taxonomy of the requested year.
#' @return A `data.frame` with 17 columns of taxonomic information: order, species code, taxon concept, common name, scientific name, family, OpenTree Taxonomy data, etc.
#' @export
#'
taxonomyGet <- function(taxonomy_year, data_path=FALSE, from_file=FALSE){
  if (from_file==FALSE){
      taxonomy_year = as.character(taxonomy_year)
        if (!is.element(taxonomy_year, clootl_data$tax_years)){
          stop("Requested taxonomy year unavailable: ", taxonomy_year)
        }
      utils::data("clootl_data")
      taxonomyYear <- paste("year", taxonomy_year, sep="")
      tax <- clootl_data$taxonomies[[`taxonomyYear`]]
  } else {
    if (data_path==FALSE || data_path==""){
        data_path = Sys.getenv('AVESDATA_PATH') ## If you didn't download it, this will be ""
    }
    if (!file.exists(data_path)){
        stop("AvesData folder not found at: ", data_path)
    }
    message("Using data download for taxonomy in")
    taxonomy_filename <- paste(data_path,
                             '/Taxonomy_versions/Clements',
                             as.character(taxonomy_year),
                             "/OTT_crosswalk_",
                             as.character(taxonomy_year),
                             ".csv",
                             sep='')
    message(taxonomy_filename)
    if (!file.exists(taxonomy_filename)){
          stop("taxonomy file not found at: ", taxonomy_filename)
          }
    tax = utils::read.csv(taxonomy_filename)
  }
  #create a convenience underscore column
  return(tax)
}

#' Helper to load a tree from a file into the R environment
#'
#' Not exported. Internal use only.
#'
#' @keywords internal
#'
treeGet <- function(version, taxonomy_year, data_path=FALSE, from_file=FALSE){
  #pull the tree file in the right version and taxonomy
  if (data_path==FALSE){
        data_path = Sys.getenv('AVESDATA_PATH') ## If you didn't download it, this will be ""
       }
  if ((data_path=="") & (from_file==TRUE)){
    stop("A data path is required to get tree from file.")
  }
  if((data_path=="") & (from_file==FALSE)){
    ## We will be in here if we have NOT run get_avesdata_repo and downloaded the data
    utils::data("clootl_data")
    taxonomyYear <- paste("year", taxonomy_year, sep="")
    version <- paste("Aves_", version, sep="")
    fullTree <- clootl_data$trees[[version]]$summary.trees[[taxonomyYear]]
  } 
  else {
  if (!file.exists(data_path)){
      stop("AvesData folder not found at: ", data_path)
  } 
    tree_filename <- paste(data_path,
                            "/Tree_versions/",
                            "Aves_",
                            version,
                            "/Clements",
                             as.character(taxonomy_year),
                             "/summary_dated_clements.nex",
                             sep='')
          message("Using data download for tree in")
          message(tree_filename)
    if (!file.exists(tree_filename)){
      stop("Tree :", tree_filename,
        "is not found. This version may not be available for this taxonomy year or you may need to update your AvesData repo using get_avesdata_repo(overwrite=TRUE)")
    } 
    else {
    fullTree <- read.nexus(tree_filename)
    fullTree$tip.label <- gsub("_", " ", fullTree$tip.label)
          }
   }

  return(fullTree)
  }

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{clootl_data}
\alias{clootl_data}
\title{A complex data store used in the package.}
\format{
List of csv files, phylogenies, and other data components.
}
\source{
\url{https://github.com/eliotmiller/clootl}
}
\usage{
clootl_data
}
\description{
A dataset containing taxonomy files, summary phylogenies, constituent study
information, and other data needed for the package to function properly.
}
\details{
The data object, \code{clootl_data}, stores the most up-to-date stable version of the
tree mapped to each of the different taxonomy years, the annotations of how each
study contributed to the tree, the citation information for each study that
contributed to the tree, the taxonomy crosswalks for different years, and
some other variables.

The structure of the data store (a list) is as follows:

\describe{
\item{\code{clootl_data$taxonomies}}{
A list of data frames. Each element corresponds to a taxonomy year:
\itemize{
\item \code{year2025}
\item \code{year2024}
\item \code{year2023}
\item \code{year2022}
\item \code{year2021}
}
These originate as CSV files linking the Clements taxonomy
for each of these years to OTT ids, Avibase ids, and other bird taxonomies
(see README of \url{https://github.com/McTavishLab/AvesData}).
}

\item{\code{clootl_data$trees}}{
\describe{
\item{\code{summary.trees}}{
Phylo objects of complete dated trees mapped to the Clements taxonomy year:
\itemize{
\item \code{year2025}
}
These are generated from \code{summary_dated_clements.nex}
(see \url{https://github.com/McTavishLab/AvesData} README).
}
\item{\code{annotations}}{
Complete annotations of the OpenTree synthetic tree for this version,
used to determine appropriate subtree citations.
}
}
}

\item{\code{clootl_data$study_info}}{
A mapping of OpenTree study ids to full citations. Used with annotations to
generate appropriate citations for trees and subtrees.
}

\item{\code{clootl_data$versions}}{
A character vector of all possible tree versions. To access older versions,
download the data repository using \code{get_avesdata_repo()}.
}

\item{\code{clootl_data$tax_years}}{
A character vector of all available taxonomies. The current tree version is
mapped to each of these taxonomies, along with crosswalks linking the Clements
taxonomy for each year to other identifiers.
}
}

This data object is generated using the following code:

\preformatted{
clootl_data = list()

clootl_data$versions <- c("1.2","1.3","1.4","1.5","1.6")
clootl_data$tax_years <- c("2021","2022","2023","2024", "2025")


clootl_data$combinations <- c(c(1.2, 2021),
                              c(1.2, 2022),
                              c(1.2, 2023),
                              c(1.3, 2021),
                              c(1.3, 2022),
                              c(1.3, 2023),
                              c(1.4, 2021),
                              c(1.4, 2022),
                              c(1.4, 2023),
                              c(1.5, 2021), <--- this will have every tip in AVONET
                              c(1.5, 2022),
                              c(1.5, 2023),
                              c(1.5, 2024),
                              c(1.6, 2025)) 

fullTree2025 <- treeGet("1.6","2025", data_path="~/projects/otapi/AvesData")
clootl_data$trees$`Aves_1.6`$summary.trees$year2025 <- fullTree2025

tax2021 <- taxonomyGet(2021, data_path="~/projects/otapi/AvesData")
tax2022 <- taxonomyGet(2022, data_path="~/projects/otapi/AvesData")
tax2023 <- taxonomyGet(2023, data_path="~/projects/otapi/AvesData")
tax2024 <- taxonomyGet(2024, data_path="~/projects/otapi/AvesData")
tax2025 <- taxonomyGet(2025, data_path="~/projects/otapi/AvesData")

clootl_data$taxonomies$year2021 <- tax2021
clootl_data$taxonomies$year2022 <- tax2022
clootl_data$taxonomies$year2023 <- tax2023
clootl_data$taxonomies$year2024 <- tax2024
clootl_data$taxonomies$year2025 <- tax2025


annot_filename <- "~/projects/otapi/AvesData/Tree_versions/Aves_1.6/OpenTreeSynth/annotated_supertree/annotations.json"
all_nodes <- jsonlite::fromJSON(txt=annot_filename)
clootl_data$trees$Aves_1.6$annotations <- all_nodes

studies <- c()
for (inputs in all_nodes$source_id_map) studies <- c(studies, inputs$study_id)
studies <- unique(studies)
study_info <- clootl:::api_studies_lookup(studies)

clootl_data$study_info <- study_info
save(clootl_data, file="~/projects/otapi/clootl/data/clootl_data.rda", compress="xz")
}
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/sampleTrees.R
\name{sampleTrees}
\alias{sampleTrees}
\title{Extract a cloud of trees from the complete Avian Phylogeny for a set of species}
\usage{
sampleTrees(
  species = "all_species",
  label_type = "scientific",
  taxonomy_year = "2025",
  version = "1.6",
  count = 100,
  data_path = FALSE
)
}
\arguments{
\item{species}{A character vector either of scientific names (directly as they come out of the
eBird taxonomy, i.e. without underscores) or of six-letter eBird species codes. Any elements of
the species vector that do not match a species-level taxon in the specified eBird taxonomy
will result in an error. Default is set to "all_species".}

\item{label_type}{Either "scientific" or "code". Default is set to "scientific".}

\item{taxonomy_year}{The eBird taxonomy year the tree should be output in. Current options
include 2021, 2022, and 2023. Both numeric and character inputs are acceptable here. Any value
aside from these years will result in an error. Default is set 2023.}

\item{version}{The desired version of the tree. Default to the most recent
version of the tree. Other versions available are listed in data object as clootl_data$versions
Verson can be passed as a character string or as numeric.}

\item{count}{Work in progress, can only sample 100 for now. Eventually: The desired number of sampled trees.}

\item{data_path}{Default to FALSE. Data_path is not necessary if the path has been stored in the
environment, as occurs when the data was downloaded using get_avesdata_repo.
Otherwise, pass in the path to the directory where the AvesData repo was downloaded.}
}
\value{
A set of phylogenies determined in \code{count} of the specified taxa in the specified eBird taxonomy version and clootl
tree version.
}
\description{
Extract a cloud of trees from the complete Avian Phylogeny for a set of species
}
\details{
This function first ensures that the requested output species overlap with species-level
taxa in the requested eBird taxonomy. If they do not, the function will error out. The onus is
on the user to ensure the requested taxa are valid. This is critical to ensure no unexpected
analysis hiccups later--you don't want to find out many steps later that your dataset doesn't
match your phylogeny. The eBird database is currently (as of Mar 2025) in 2024 taxonomy.
Trees available in 2024 taxonomy will be available by June 2025. The 2025 taxonomy will be released
to the public in October or November 2025. The intention is to release a tree in 2025 taxonomy
concurrently with the publication of the taxonomy itself.
}
\examples{
if (Sys.getenv("AVESDATA_PATH") != "") {
  ex2 <- sampleTrees(species=c("Turdus migratorius",
                             "Setophaga dominica",
                             "Setophaga ruticilla",
                             "Sitta canadensis"))
 }
}
\author{
Eliot Miller, Luna Sanchez Reyes, Emily Jane McTavish
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/extractTree.R
\name{treeGet}
\alias{treeGet}
\title{Helper to load a tree from a file into the R environment}
\usage{
treeGet(version, taxonomy_year, data_path = FALSE, from_file = FALSE)
}
\description{
Not exported. Internal use only.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils.R
\name{set_avesdata_repo_path}
\alias{set_avesdata_repo_path}
\title{Set path to Aves Data folder}
\usage{
set_avesdata_repo_path(path, overwrite = FALSE, warn = TRUE)
}
\arguments{
\item{path}{A character vector with the path to the Aves Data folder}

\item{overwrite}{Boolean, default to \code{FALSE}, does not overwrite an existing Aves Data folder. Set to \code{TRUE} to overwrite.}

\item{warn}{Boolean, default to \code{TRUE}, Warns if path does not exist.
Set to FALSE and path="" to unset path.}
}
\value{
No return value, called to set the path to the Aves Data folder.
}
\description{
Set path to Aves Data folder already somewhere on your computer
}
\details{
Based on https://github.com/CornellLabofOrnithology/auk/blob/main/R/auk-set-ebd-path.r
Use this function to manually set or update location of a downloaded AvesData folder from https://github.com/McTavishLab/AvesData.
When \code{AVESDATA_PATH} is set, the data_path in any clootl functions with a \code{data_path} argument will default to this value.
}
\examples{
\dontrun{
set_avesdata_repo_path("/home/ejmctavish/AvesData")
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils.R
\name{get_avesdata_repo_path}
\alias{get_avesdata_repo_path}
\title{Get path to Aves Data folder}
\usage{
get_avesdata_repo_path()
}
\value{
String - path to Aves Data folder, if set. Returns "" if not set.
}
\description{
Get path to Aves Data folder, if set.
}
\details{
Based on https://github.com/CornellLabofOrnithology/auk/blob/main/R/auk-set-ebd-path.r
Use this function to check stored path to downloaded AvesData folder from https://github.com/McTavishLab/AvesData.
When \code{AVESDATA_PATH} is set, the data_path in any clootl functions with a \code{data_path} argument will default to this value.
}
\examples{
\dontrun{
get_avesdata_repo_path()
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils.R
\name{get_avesdata_repo}
\alias{get_avesdata_repo}
\title{Download the AvesData full repository}
\usage{
get_avesdata_repo(path, overwrite = FALSE)
}
\arguments{
\item{path}{Path to download data zipfile to, and where it will be unpacked.  To download into your working directory, use "."}

\item{overwrite}{Default to \code{FALSE}. If path exists, will not re-download the data unless overwrite=\code{TRUE}.}
}
\value{
No return value. This function is used to download the Aves Data repository.
}
\description{
Pull down full AvesData repository to a working directory
}
\details{
Will download full data repo from https://github.com/McTavishLab/AvesData.
This data is required to use \code{\link[=sampleTrees]{sampleTrees()}} to sample from the distribution of dated trees,
or to access earlier versions of the complete tree.
This function will download the data and set an environmental variable \code{AVESDATA_PATH} to the location of the data download.
When \code{AVESDATA_PATH} is set, the data_path in any clootl functions with a \code{data_path} argument will default to this value.
To manually set \code{AVESDATA_PATH} to the location of your downloaded AvesData repo use \code{\link[=set_avesdata_repo_path]{set_avesdata_repo_path()}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/extractTree.R
\name{taxonomyGet}
\alias{taxonomyGet}
\title{Load a bird taxonomy into the R environment}
\usage{
taxonomyGet(taxonomy_year, data_path = FALSE, from_file = FALSE)
}
\arguments{
\item{taxonomy_year}{The eBird taxonomy year the tree should be output in. Current options
are 2021-2024. Both numeric and character inputs are acceptable here. Any value
aside from these years will result in an error. Default is most recent year.}

\item{data_path}{Default to FALSE. If a summary, dated tree is desired, this is sufficient
and does not need to be modified. However, if a user wishes to extract a set of complete
dated trees, for example to iterate an analysis across a cloud of trees, or to use an
older version of the tree than the current one packed in the data object, this function
can also accept a path to the downloaded set of trees. If you have already downloaded the AvesData repo
available at https://github.com/McTavishLab/AvesData use data_path= the path to the download location.
Alternately, you can download the full data repo using \code{\link[=get_avesdata_repo]{get_avesdata_repo()}}. This approach will download the data and
set an environmental variable AVESDATA_PATH. When AVESDATA_PATH is set, the data_path will default to this value.
To manually set AVESDATA_PATH to the location of your downloaded AvesData repo use \code{\link[=set_avesdata_repo_path]{set_avesdata_repo_path()}}}

\item{from_file}{Default to FALSE. If TRUE forces taxonomyGet to use a local copy of the the taxonomy.
This is useful for testing changes and/or updating the clootl_data object.}
}
\value{
A \code{data.frame} with 17 columns of taxonomic information: order, species code, taxon concept, common name, scientific name, family, OpenTree Taxonomy data, etc.
}
\description{
\code{taxonomyGet} either reads a taxonomy file and loads it
as a \verb{data frame}, or loads the default taxonomy data object.
}
\details{
This will return a data object that has the taxonomy of the requested year.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/extractTree.R
\name{extractTree}
\alias{extractTree}
\title{Extract a complete or pre-pruned phylogeny from the clootl datastore}
\usage{
extractTree(
  species = "all_species",
  label_type = "scientific",
  taxonomy_year = 2025,
  version = "1.6",
  data_path = FALSE,
  force = FALSE
)
}
\arguments{
\item{species}{A character vector either of scientific names (directly as they come out of the
eBird taxonomy, i.e. without underscores) or of six-letter eBird species codes. Any elements of
the species vector that do not match a species-level taxon in the specified eBird taxonomy
will result in an error. eBird taxonomy files can be accessed using \code{\link[=taxonomyGet]{taxonomyGet()}}. Default is set to "all_species".}

\item{label_type}{Either "scientific" or "code". Default is set to "scientific".}

\item{taxonomy_year}{The eBird taxonomy year the tree should be output in. Current options
are 2021-2024. Both numeric and character inputs are acceptable here. Any value
aside from these years will result in an error. Default is most recent year.}

\item{version}{The desired version of the tree. Default to the most recent
version of the tree. Other versions available are listed in clootl_data$versions and can be
passed as a character string or as numeric.}

\item{data_path}{Default to FALSE. If a summary, dated tree is desired, this is sufficient
and does not need to be modified. However, if a user wishes to extract a set of complete
dated trees, for example to iterate an analysis across a cloud of trees, or to use an
older version of the tree than the current one packed in the data object, this function
can also accept a path to the downloaded set of trees. If you have already downloaded the AvesData repo
available at https://github.com/McTavishLab/AvesData use data_path= the path to the download location.
Alternately, you can download the full data repo using \code{\link[=get_avesdata_repo]{get_avesdata_repo()}}. This approach will download the data and
set an environmental variable AVESDATA_PATH. When AVESDATA_PATH is set, the data_path will default to this value.
To manually set AVESDATA_PATH to the location of your downloaded AvesData repo use \code{\link[=set_avesdata_repo_path]{set_avesdata_repo_path()}}}

\item{force}{Default to FALSE.  If FALSE a tree will be returned only if there is an exact match in the tree
to all species requested. This function first ensures that the requested output species overlap with species-level
taxa in the requested eBird taxonomy. If they do not, the function will error out. The onus is
on the user to ensure the requested taxa are valid. This is critical to ensure no unexpected
analysis hiccups later--you don't want to find out many steps later that your dataset doesn't
match your phylogeny.
If force=TRUE even if there is is not a match to all taxa in the requested species list, a tree will be
returned for the species that do match.}
}
\value{
One or more phylogenies of the specified taxa in the specified eBird taxonomy version and clootl
tree version.
}
\description{
This function extracts one or more phylogenies in the desired taxonomy and tree version.
It defaults to the pre-packaged summary trees, but can also be used to extract sets of
phylogenies expressing uncertainty, once they have been downloaded from the online repository.
}
\examples{
ex1 <- extractTree(species=c("amerob", "canwar", "reevir1", "yerwar", "gockin"),
   label_type="code")
ex2 <- extractTree(species=c("Turdus migratorius",
                             "Setophaga dominica",
                             "Setophaga ruticilla",
                             "Sitta canadensis"),
   label_type="scientific",
   taxonomy_year="2025",
   version="1.6")

}
\author{
Eliot Miller, Luna Sanchez Reyes, Emily Jane McTavish
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/getCitations.R
\name{getCitations}
\alias{getCitations}
\title{Identify contributing studies}
\usage{
getCitations(tree, version = 1.6, data_path = FALSE)
}
\arguments{
\item{tree}{A phylogeny obtained from extractTree (see details).}

\item{version}{The version of the tree used in extract tree. Default to the most recent
version of the tree. and can be passed as a character string or as numeric. If an alternate
version was used to create the tree this function may fail or give incomplete or incorrect
citation information.}

\item{data_path}{Default to FALSE. If you are gathering citations for an
older version of the tree than the current one packed in the data object, you will have
already downloaded the data repo in order to generate that tree.
The data is available at https://github.com/McTavishLab/AvesData.
If you have manually downloaded the repo, use data_path= the path to the download location.
Alternately, you can download the full data repo using \code{\link[=get_avesdata_repo]{get_avesdata_repo()}}.
This approach will download the data and set an environmental variable AVESDATA_PATH.
When AVESDATA_PATH is set, the data_path argument will default to this value.
To manually set AVESDATA_PATH to the location of your downloaded AvesData repo
use \code{\link[=set_avesdata_repo_path]{set_avesdata_repo_path()}}}
}
\value{
A dataframe of the percent of internal nodes supported by a given study, as well
as the DOI of that study. The proportion of taxa in the tree supported by taxonomic
addition only is also included in the dataframe.
}
\description{
Quantify the contribution of studies informing an extracted tree,
and obtain DOI and citation information for those studies.
}
\details{
The function will determine
what proportion of nodes in your phylogeny are supported by each study that goes into
creating the final clootl tree. We use 'supported by' in the sense described in
Redelings and Holder, PeerJ (2017) https://peerj.com/articles/3058/, and as shown in the
tree.opentreeoflife.org tree viewer. We normalize these values to
a percentage of internal nodes in the target tree supported by each study.
In any resulting publication, please cite both the
synthetic tree (McTavish et al. 2025), clootl (Miller et al. 2025) and
"all" the trees/DOIs that contributed to
your phylogeny. That said, we are well aware of citation and word count
limits that plague modern publishing, and for this reason we quantify the
contribution of each study; depending on your phylogeny,
it is very possible that one or two studies contributed the majority of information.
This function relies on the phylogenetic synthesis information directly,
and is agnostic to taxonomy version.
}
\examples{
\donttest{#pull the taxonomy file out
data(clootl_data)
tax <- clootl_data$taxonomies$year2025
ls(tax)
#simulate extracting a tree for a particular family
temp <- tax[tax$FAMILY=="Rhinocryptidae (Tapaculos)",]
spp <- temp$SCI_NAME

#get your tree
prunedTree <- extractTree(species=spp, label_type="scientific",
   taxonomy_year=2025, version="1.6")

#get your citation DF
yourCitations <- getCitations(tree=prunedTree)}
}
\author{
Eliot Miller, Emily Jane McTavish
}
